(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: walker_ast.mli,v 1.15 2014/01/31 09:51:52 fclement Exp $ *)

(** Abstract syntax of function definitions that walk on a type
  definition. *)

type file = phrase list

and phrase =
  | Comment of string
  | Directive of dir
  | Type_definition of rec_flag * type_defs

and rec_flag =
  | Rec
  | No_rec

and dir =
  | Dir_open of string
  | Dir_include of string

and type_defs = type_def list

and type_def =
  (Og_types.type_name * walker_name) * poly_arguments * type_kind

and walker_name = string

and poly_arguments = name list

and name = string

and type_kind =
  | Record of buff_argument * Og_types.visibility * record_clause
  | Sum of buff_argument * Og_types.visibility * sum_clause
  | Alias of buff_argument * Og_types.visibility * alias_clause
  | External of buff_argument * external_clause

and buff_argument = pattern option

and record_clause = record_field list
and record_field =
  | Record_field of record_label * function_clause

and record_label = string

and sum_clause = sum_field list
and sum_field =
  | Sum_tag of sum_label
  | Sum_inj of sum_label * function_clause

and sum_label = string

and alias_clause = function_clause * type_kind option

and external_clause = function_clause

and function_clause =
  | Clause of pattern * expr

and pattern =
  | Pvariable of name * Og_types.type_expr
  | Ptuple of pattern array * Og_types.type_expr

and function_name =
    (* Argument of a functional walk *)
  | Fname_arg of name
    (* Named walk *)
  | Fname_local of name
    (* External walk *)
  | Fname_external of name

and function_expr =
  | Fconstant of function_name
  | Fapply of function_name * function_expr list
  | Fabs of pattern list * expr
  | Fparen of function_expr

and expr =
    (* Defining an alias for a function expression
  | Econstant of function_expr*)
    (* Walk application *)
  | Eapply of function_expr * argument list
    (* Tuples walking *)
  | Etuple of expr array

and argument =
  | Avar of pattern
  | Ainj of function_name * pattern
;;
