(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_debug.ml,v 1.2 2014/01/31 09:51:52 fclement Exp $ *)

(** {3 Some simple generic tools for debugging} *)

let get_debug, set_debug =
  let debug = ref false in
  (fun () -> !debug),
  (fun () -> debug := true)
;;

(** A special [Debug.failwith] function to be able to trace the failure
  strings. *)
let failwith failure_string =
  if get_debug () then prerr_endline failure_string;
  raise (Failure failure_string)
;;

let printf fmt =
  if get_debug ()
  then Printf.fprintf stderr fmt
  else Printf.ifprintf stderr fmt
;;

let warning s =
  Printf.fprintf stderr "%s: warning, %s\n"
    (Filename.basename Sys.executable_name)
    s
;;

let message s =
  if get_debug () then prerr_endline s else ()
;;
